//Resources
var sprites = new Image();
sprites.src = "gfx/sprite.png";
var back = new Image();
back.src = "gfx/back.png";
var logo = new Image();
logo.src = "gfx/logo.png";
var sfx = {
	jump: new Audio("sound/jump.mp3"),
	powerup: new Audio("sound/powerup.mp3"),
	win: new Audio("sound/win.mp3"),
	apple: new Audio("sound/apple.mp3"),
	blockdestroy: new Audio("sound/blockdestroy.mp3"),
	blockhit: new Audio("sound/blockhit.mp3"),
	blocks: new Audio("sound/blocks.mp3"),
	explode: new Audio("sound/explode.mp3"),
	lose: new Audio("sound/lose.mp3"),
	jetpack: new Audio("sound/jetpack.mp3"),
	hurt: new Audio("sound/hurt.mp3"),
};
for(var i in sfx) {
	sfx[i].load();
}

//Globals
var keystate = [];
var gridsize = 32;
var canvassize = {
	w: 800,
	h: 600
};
var room = {
	w: 768,
	h: 8096
};
var roomgrid = {
	x: room.w / gridsize,
	y: room.h / gridsize
};
var cameralimit = {
	x: room.w - canvassize.w + gridsize,
	y: room.h - canvassize.h + gridsize
};
var canvashalf = {
	w: canvassize.w / 2,
	h: canvassize.h / 2
};
var player = {};
var camera = {};
var secondstimeout;
var keyevent;
var textopacity = 0;

//Blocks
var blocks = [];
var blocklist = {};
function block(x, y, type, timer, i) {
	i = i || {};
	i.x = x * gridsize;
	i.y = y * gridsize;
	i.type = type;
	i.timer = 0;
	i.draw = function() {
		var offset = 0;
		switch (this.type) {
			case "spike":
				offset = 8;
				break;
			case "boundary":
				offset = 16;
				break;
			case "break":
				offset = 24;
				break;
		}
		gameport.drawImage(sprites, offset, 16, 8, 8, this.x - camera.x, this.y - camera.y, gridsize, gridsize);
	};
	return i;
}

//Animations (power-ups)
var animations = [];
function animation(x, y, type, i) {
	i = i || {};
	i.x = x;
	i.y = y;
	i.type = type;
	i.opacity = 1;
	i.draw = function() {
		this.opacity = Math.round((this.opacity - 0.04) * 100) / 100;
		this.y-=4;
		gameport.globalAlpha = this.opacity;
		var offset = 0;
		switch (this.type) {
			case "explosion":
				offset = 8;
				break;
			case "jetpack":
				offset = 16;
				break;
			case "blocks":
				offset = 24;
				break;
		}
		gameport.drawImage(sprites, offset, 8, 8, 8, this.x - camera.x, this.y - camera.y, gridsize, gridsize);
		gameport.globalAlpha = 1;
	};
	return i;
}

//Particles
var particles = [];
function particle(x, y, r, g, b, type, i) {
	i = i || {};
	i.x = x + Math.floor(Math.random() * 4 + -2) * 4 + 16;
	i.y = y + Math.floor(Math.random() * 4 + -2) * 4 + 16;
	i.r = r;
	i.g = g;
	i.b = b;
	i.type = type;
	i.size = Math.floor(Math.random() * 2 + 1) * 4;
    i.vx = Math.random() * 4 - 2;
    i.vy = Math.random() * -2 - 3;
	i.opacity = 1;
	i.draw = function() {
		this.opacity = Math.round((this.opacity - 0.025) * 100) / 100;
        this.x += this.vx;
		if (!type) {
			this.vy += 0.2;
			this.y += this.vy;
		} else {
			this.y += -this.vy;
		}
        gameport.fillStyle = "rgba(" + this.r + ", " + this.g + ", " + this.b + ", " + this.opacity + ")";
        gameport.fillRect(this.x - camera.x, this.y - camera.y, this.size, this.size);
	};
	return i;
}

function pushparticles(x, y, r, g, b, amount, type) {
	if (!amount) {
		amount = 5;
	}
	for(var i = 0; i < amount; i++) {
		particles.push(particle(x, y, r, g, b, type));
	}
}

//Items
var items = [];
function item(x, y, type, i) {
	i = i || {};
	i.x = x * gridsize;
	i.y = y * gridsize;
	i.type = type;
	i.draw = function() {
		gameport.drawImage(sprites, 0, 8, 8, 8, this.x - camera.x, this.y - camera.y, gridsize, gridsize);
	};
	return i;
}

function init() {

	player = {
		x: gridsize * 12,
		y: gridsize * 3,
		w: 28,
		h: 28,
		speed: 8,
		jump: false,
		jumptoggle: 0,
		jumpheight: 0,
		animation: 0,
		apples: 20,
		timer: 0,
		hurt: false,
		jetpack: 0,
		screenhold: -1,
		start: false,
		items: {
			explosion: true,
			jetpack: true,
			blocks: true
		},
		velocity: {
			x: 0,
			y: 0
		},
		hunger: 10,
		onground: false,
		draw: function() {
			var offset = 0;
			if (player.start) {
				//Running animations
				if (keystate[68] || keystate[39]) {
					offset = 16;
				} else if (keystate[65] || keystate[37]) {
					offset = 32;
				}
				if (player.velocity.x >= 1) {
					offset = 16;
					if (player.animation % 4 === 0) {
						offset = 8;
					}
					if (!player.onground) {
						offset = 16;
					}
				} else if (player.velocity.x <= -1) {
					offset = 32;
					if (player.animation % 4 === 0) {
						offset = 24;
					}
					if (!player.onground) {
						offset = 32;
					}
				}
				//Hurt sprite
				if (player.hurt) {
					offset = 40;
				}
			}
			//Draw fire under player
			if (player.jetpack) {
				gameport.drawImage(sprites, 16, 8, 8, 8, this.x - 4 - camera.x, this.y + 20 - camera.y, gridsize, gridsize);
			}
			gameport.drawImage(sprites, offset, 0, 8, 8, this.x - 4 - camera.x, this.y - 4 - camera.y, gridsize, gridsize);
		}
	};

	//Resets
	camera = {
		x: 0,
		y: 0
	};
	animations = [];
	particles = [];

    background.fillStyle = background.createPattern(back, 'repeat');
    background.fillRect(0, 0, background.canvas.width, background.canvas.height);

	blocks = [];
	blocklist = {};
	//Block patterns
	for(var x = 0; x <= roomgrid.x; x++) {
		for(var y = 0; y <= roomgrid.y; y++) {
			if (x === 0) {
				pushblock(x, y, "boundary", true);
			} else if (y === 0) {
				pushblock(x, y, "boundary", true);
			} else if (x == roomgrid.x) {
				pushblock(x, y, "boundary", true);
			} else if (y == roomgrid.y) {
				pushblock(x, y, "boundary", true);
			} else if (x == 12 && y == 8 || x == 11 && y == 8 || x == 13 && y == 8) {
				pushblock(x, y, "boundary", true);
			} else if (y >= 10) {
				if (Math.floor((Math.random() * 45 ) + 1) == 1) {
					pushblock(x-2, y);
					pushblock(x-1, y);
					pushblock(x, y);
					pushblock(x+1, y);
					pushblock(x+2, y);
					if (Math.floor((Math.random() * 20 ) + 1) == 1) {
						pushblock(x-1, y-4, "break");
						pushblock(x+1, y-4, "break");
					}
					if (Math.floor((Math.random() * 12 ) + 1) == 1) {
						pushblock(x-3, y, "break");
						pushblock(x+3, y, "break");
					}
					if (Math.floor((Math.random() * 25 ) + 1) == 1) {
						pushblock(x-4, y-1, "break");
						pushblock(x+4, y-1, "break");
					}
					if (Math.floor((Math.random() * 25 ) + 1) == 1) {
						pushblock(x-4, y+1, "break");
						pushblock(x+4, y+1, "break");
					}
					if (Math.floor((Math.random() * 3 ) + 1) == 1) {
						pushblock(x, y-1, "spike");
					}
					if (Math.floor((Math.random() * 10 ) + 1) == 1) {
						pushblock(x-1, y-1, "spike");
						pushblock(x+1, y-1, "spike");
					}
					if (Math.floor((Math.random() * 40 ) + 1) == 1) {
						pushblock(x-2, y-5, "break");
						pushblock(x-1, y-5, "break");
						pushblock(x, y-5, "break");
						pushblock(x+1, y-5, "break");
						pushblock(x+2, y-5, "break");
					}
				}
			}
		}
	}

	//Spawn apples
	items = [];
	var itemcount = 0;
	while (itemcount < 20) {
		var randomx = Math.floor(Math.random() * roomgrid.x);
		var randomy = Math.floor(Math.random() * roomgrid.y);
		if (blocklist[randomx] && !blocklist[randomx][randomy]) {
			items.push(item(randomx, randomy));
			itemcount++;
		}
	}

}

//Make sure we can place a block there
function pushblock(x, y, type, bypass) {
	blocklist[x] = blocklist[x] || {};
	if (!blocklist[x][y] && x !== 0 && y !== 0 && x !== roomgrid.x && y !== roomgrid.y || bypass) {
		blocklist[x][y] = true;
		blocks.push(block(x, y, type));
	}
}

window.requestAnimFrame = (function(){
	return window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame || window.oRequestAnimationFrame || window.msRequestAnimationFrame || function(callback){ window.setTimeout(callback, 1000 / 60); };
})();

//Hunger clock
function seconds() {
	textopacity = 1;
	player.hunger--;
	player.timer++;
	secondstimeout = setTimeout(function(){
		seconds();
	}, 1000);
}

function rendering() {
	//setTimeout(rendering, 150); //For debug
	requestAnimFrame(rendering);
	update();
}

function lerp(a, b, f) {
	//Linear interpolation for camera smoothing with speed bitwise shift rounding hack: (0.5 + somenumber) << 0 SUPER FAST!
    return (0.5 + (a + f * (b - a))) << 0;
}

function flashbackground(type) {
	if (type) {
		background.fillStyle = type;
	} else {
		background.fillStyle = background.createPattern(back, 'repeat');
	}
	background.fillRect(0, 0, background.canvas.width, background.canvas.height);
}

function update() {

	//Spacebar start (death/logo screen)
	if (keystate[32] && !player.start) {
		init();
		seconds();
		player.start = true;
		keystate[32] = 0;
	}

	if (!player.start) {
		return false;
	}

	//Redraw UI
	ui.clearRect(0, 0, canvassize.w, canvassize.h);
	if (player.hunger !== 10) {
		//Large fading numbers
		if (textopacity > 0) {
			ui.globalAlpha = textopacity;
			if (player.hunger <= 3) {
				//Shift 3, 2, 1 to middle of screen
				ui.drawImage(sprites, player.hunger * 8, 24, 8, 8, 352, 252, 128, 128);
			} else {
				ui.drawImage(sprites, player.hunger * 8, 24, 8, 8, 0, 0, 128, 128);
			}
			ui.globalAlpha = 1;
		}
		textopacity = Math.round((textopacity - 0.05) * 100) / 100;

		ui.drawImage(sprites, player.hunger * 8, 24, 8, 8, 12, 12, 32, 32);
	} else {
		//10 seconds
		ui.drawImage(sprites, 8, 24, 8, 8, 12, 12, 36, 32);
		ui.drawImage(sprites, 0, 24, 8, 8, 44, 12, 36, 32);
	}

	//HUD
	ui.drawImage(sprites, 0, 8, 8, 8, 700, 12, 32, 32);
	var apple1 = player.apples.toString().substr(0, 1);
	var apple2 = player.apples.toString().substr(1, 1);
	ui.drawImage(sprites, parseInt(apple1, 10) * 8, 24, 8, 8, 736, 12, 32, 32);
	ui.drawImage(sprites, parseInt(apple2, 10) * 8, 24, 8, 8, 764, 12, 32, 32);

	//Items HUD
	ui.drawImage(sprites, 8, 24, 8, 8, 12, 480, 32, 32); //1
	if (player.items.explosion) {
		ui.drawImage(sprites, 8, 8, 8, 8, 36, 480, 32, 32);
	}
	ui.drawImage(sprites, 16, 24, 8, 8, 8, 520, 32, 32); //2
	if (player.items.jetpack) {
		ui.drawImage(sprites, 16, 8, 8, 8, 36, 520, 32, 32);
	}
	ui.drawImage(sprites, 24, 24, 8, 8, 12, 560, 32, 32); //3
	if (player.items.blocks) {
		ui.drawImage(sprites, 24, 8, 8, 8, 36, 560, 32, 32);
	}

	//DEAD!
	if (player.hunger <= 0) {
		sfx.lose.play();
		player.start = false;
		flashbackground("#ff0000");
		player.screenhold = 10;
		ui.drawImage(sprites, 32, 16, 48, 8, 208, 112, 384, 64); //U BE DEAD YO
		ui.drawImage(sprites, 32, 8, 40, 8, 236, 448, 328, 64); //Spacebar
		clearTimeout(secondstimeout);
	}

	//WIN
	if (player.apples <= 0) {
		sfx.win.play();
		player.start = false;
		player.screenhold = 10;
		flashbackground("#a100ff");
		ui.drawImage(sprites, 0, 32, 72, 8, 112, 48, 576, 64); //Pro Strats
		ui.drawImage(sprites, 0, 40, 30, 8, 112, 128, 240, 64); //Time

		//Convert second timer into graphics
		var len = player.timer.toString().length;
		var time = [];
		for (var i = 0; i < len; i++) {
			time[i] = player.timer.toString().substr(i, 1);
			time[i] = parseInt(time[i], 10);
			ui.drawImage(sprites, time[i] * 8, 24, 8, 8, 368 + (64 * i), 136, 64, 64);
		}

		ui.drawImage(sprites, 32, 8, 40, 8, 236, 448, 328, 64); //Spacebar
		clearTimeout(secondstimeout);
	}

	//Background screen effects
	if (player.screenhold > 1) {
		player.screenhold--;
	} else if (player.screenhold > 0) {
		player.screenhold--;
		flashbackground();
		player.hurt = false;
	}

	gameport.clearRect(0, 0, canvassize.w, canvassize.h);

	//Movement based on velocity
	if (player.velocity.x >= 1) {
		player.x+=Math.floor(player.velocity.x * player.velocity.x / 75);
		collisions("right");
	} else if (player.velocity.x <= -1) {
		player.x-=Math.floor(player.velocity.x * player.velocity.x / 75);
		collisions("left");
	}

	//Powerups
	if (player.screenhold <= 0) {
		//1
		if (player.items.explosion && keystate[49]) {
			sfx.explode.play();
			pushparticles(player.x, player.y, 255, 255, 255, 15);
			player.items.explosion = false;
			flashbackground('#ffffff');
			player.screenhold = 10;
			//Make a square radius around the player to delete blocks around him
			for(var i in blocks) {
				if (blocks[i].type !== "boundary") {
					var explosion = {
						x: player.x - gridsize * 4,
						y: player.y - gridsize * 4,
						w: gridsize * 8,
						h: gridsize * 8
					};
					while (collides("blocks", i, explosion)) {
						delete blocks[i];
						break;
					}
				}
			}
		}
		//2
		else if (player.items.jetpack && keystate[50]) {
			sfx.jetpack.play();
			flashbackground('#ff9c00');
			player.screenhold = 10;
			player.items.jetpack = false;
			player.jetpack = 200;
			player.jump = false;
		}
		//3
		else if (player.items.blocks && keystate[51]) {
			sfx.blocks.play();
			flashbackground('#0030ff');
			player.screenhold = 10;
			player.items.blocks = false;

			var playerx = Math.floor(player.x / gridsize) * gridsize / gridsize;
			var playery = Math.floor(player.y / gridsize) * gridsize / gridsize;

			pushblock(playerx-2, playery-2);
			pushblock(playerx-1, playery-2);
			pushblock(playerx, playery-2);
			pushblock(playerx+1, playery-2);
			pushblock(playerx+2, playery-2);

		}
	}

	//Right
	if (keystate[68] || keystate[39]) {
		player.x+=Math.floor(player.speed / 2);
		collisions("right");
		player.animation++;
		if (player.onground) {
			player.velocity.x++;
		}
		//Max velocity
		if (player.velocity.x >= 20) {
			player.velocity.x = 20;
		} else if (player.velocity.x <= -1) {
			player.velocity.x = 0;
		}
	} else if (keystate[65] || keystate[37]) {
	//Left
		player.x-=Math.floor(player.speed / 2);
		collisions("left");
		player.animation++;
		if (player.onground) {
			player.velocity.x--;
		}
		//Max velocity
		if (player.velocity.x <= -20) {
			player.velocity.x = -20;
		} else if (player.velocity.x >= 1) {
			player.velocity.x = 0;
		}
	} else {
		//Friction
		if (player.velocity.x >= 1) {
			player.velocity.x--;
			if (player.onground && player.velocity.x >= 10) {
				//Stop friction if player is moving other direction
				if (!keystate[65] && !keystate[37]) {
					player.x+=Math.floor(player.speed / 2);
					collisions("right");
				}
			}
		} else if (player.velocity.x <= -1) {
			player.velocity.x++;
			if (player.onground && player.velocity.x <= -10) {
				//Stop friction if player is moving other direction
				if (!keystate[68] && !keystate[39]) {
					player.x-=Math.floor(player.speed / 2);
					collisions("left");
				}
			}
		}
	}

	if (player.jetpack <= 0) {
		if (!player.jump) {
			//Falling
			player.onground = false;
			player.velocity.y--;
			//Terminal velocity
			if (player.velocity.y <= -30) {
				player.velocity.y = -30;
			}
			player.y += player.speed + Math.floor(-player.velocity.y * -player.velocity.y / 100);
			collisions("down");
			//Jump
			if (keystate[87] || keystate[38] || keystate[32]) {
				//Allow the player to jump if they are on the ground, or just started falling (to simulate mario style corner jumping)
				if (player.onground && player.jumptoggle === 0 || player.velocity.y > -4 && player.jumptoggle === 0) {
					player.jump = true;
					player.jumptoggle = 1;
					player.onground = false;
					player.jumpheight = 0;
					sfx.jump.play();
				}
			}
		} else {
			//Jumping
			player.y -= player.speed;
			collisions("up");

			//Arc at top of jump - this inadvertently also allow you to clip up edges up blocks if on the very edge (intentional)
			player.y += Math.floor(player.jumpheight * player.jumpheight / 100);
			collisions("down");

			player.jumpheight++;

			if (keystate[87] || keystate[38] || keystate[32]) {
				if (player.jumpheight >= 40) {
					player.jump = false;
				}
			} else {
				//Don't allow super tiny jumps
				if (player.jumpheight >= 5) {
					player.jump = false;
				}
			}
		}
	} else {
		//Jetpacking item
		player.jetpack--;
		player.y -= player.speed;
		collisions("up");
		//Draw particles on jetpack
		pushparticles(player.x - 4, player.y + gridsize + 4, 255, 255, 255, 2, "jetpack");
	}

	//Camera limits/locks
	//Allow 32 pixels of threshold x movement
	if (camera.x >= player.x - canvashalf.w + gridsize || camera.x <= player.x - canvashalf.w - gridsize) {
		camera.x = lerp(camera.x, player.x - canvashalf.w, 0.05);
		if (camera.x < 0) {
			camera.x = 0;
		} else if (camera.x > cameralimit.x) {
			camera.x = cameralimit.x;
		}
	}

	//Only shift when player is on a platform or has certain velocity, or player is jumping
	if (player.onground || player.velocity.y <= -10 || player.jump || player.jetpack) {
		camera.y = lerp(camera.y, player.y - canvashalf.h, 0.1);
		if (camera.y < 0) {
			camera.y = 0;
		} else if (camera.y > cameralimit.y) {
			camera.y = cameralimit.y;
		}
	}

	render();

}

function render() {
	player.draw();
	for(var i in blocks) {
		//Don't draw out of frame
		if(blocks[i].x-(canvassize.w+gridsize) > camera.x || blocks[i].x+gridsize - camera.x < 0 ) continue;
		if(blocks[i].y-(canvassize.h+gridsize) > camera.y || blocks[i].y+gridsize - camera.y < 0 ) continue;
		blocks[i].draw();
	}
	for(var i2 in items) {
		if(items[i2].x-(canvassize.w+gridsize) > camera.x || items[i2].x+gridsize - camera.x < 0 ) continue;
		if(items[i2].y-(canvassize.h+gridsize) > camera.y || items[i2].y+gridsize - camera.y < 0 ) continue;
		items[i2].draw();
	}
	var animationcount = 0;
	for(var i3 in animations) {
		if(animations[i3].x-(canvassize.w+gridsize) > camera.x || animations[i3].x+gridsize - camera.x < 0 ) continue;
		if(animations[i3].y-(canvassize.h+gridsize) > camera.y || animations[i3].y+gridsize - camera.y < 0 ) continue;
		if(animations[i3].opacity <= 0) continue;
		animationcount++;
		animations[i3].draw();
	}
	//Remove animations after they are all gone
	if (animationcount === 0) {
		animations = [];
	}
	var particlescount = 0
	for(var i4 in particles) {
		if(particles[i4].x-(canvassize.w+gridsize) > camera.x || particles[i4].x+gridsize - camera.x < 0 ) continue;
		if(particles[i4].y-(canvassize.h+gridsize) > camera.y || particles[i4].y+gridsize - camera.y < 0 ) continue;
		if(particles[i4].opacity <= 0) continue;
		particlescount++;
		particles[i4].draw();
	}
	//Remove particles after they are all gone
	if (particlescount === 0) {
		particles = [];
	}
}

function randomitem(x, y) {
	if (player.hunger < 10) {
		player.hunger++;
	}

	if (!player.items.jetpack && Math.floor((Math.random() * 4 ) + 1) == 1) {
		player.items.jetpack = true;
		animations.push(animation(x, y, "jetpack"));
		sfx.powerup.play();
		return true;
	}
	if (!player.items.blocks && Math.floor((Math.random() * 4 ) + 1) == 1) {
		player.items.blocks = true;
		animations.push(animation(x, y, "blocks"));
		sfx.powerup.play();
		return true;
	}
	if (!player.items.explosion && Math.floor((Math.random() * 4 ) + 1) == 1) {
		player.items.explosion = true;
		animations.push(animation(x, y, "explosion"));
		sfx.powerup.play();
		return true;
	}
}

function collisions(dir) {
	var breakfor = false;
	for(var i in blocks) {
		while (collides("blocks", i, player)) {
			breakfor = false;
			switch(dir) {
				case "left":
					player.x+=1;
					player.velocity.x = 0;
					break;
				case "right":
					player.x-=1;
					player.velocity.x = 0;
					break;
				case "down":
					if (blocks[i].type == "break") {
						blocks[i].timer++;
						player.y-=1;
						player.onground = true;
						player.velocity.y = 0;
						if (blocks[i].timer >= 250) {
							randomitem(blocks[i].x, blocks[i].y);
							sfx.blockdestroy.play();
							pushparticles(blocks[i].x, blocks[i].y, 255, 138, 0, 5, "downbreak");
							delete blocks[i];
							breakfor = true;
						}
					} else if (blocks[i].type == "spike") {
						player.hunger--;
						player.screenhold = 10;
						player.hurt = true;
						pushparticles(player.x - 16, player.y + gridsize, 255, 0, 0);
						sfx.hurt.play();
						flashbackground("#ff0000");
						delete blocks[i];
						breakfor = true;
					} else {
						player.y-=1;
						player.onground = true;
						player.velocity.y = 0;
					}
					if (!keystate[87] && !keystate[38] && !keystate[32]) {
						player.jumptoggle = 0;
					}
					break;
				case "up":
					if (blocks[i].type == "break") {
						randomitem(blocks[i].x, blocks[i].y);
						sfx.blockdestroy.play();
						pushparticles(blocks[i].x, blocks[i].y, 255, 138, 0);
						delete blocks[i];
						breakfor = true;
					} else {
						sfx.blockhit.play();
					}
					player.y+=1;
					player.jump = false;
					player.jumptimer = 0;
				break;
			}
			if (breakfor) {
				break;
			}
		}
	}
	for(var i in items) {
		while (collides("items", i, player)) {
			pushparticles(items[i].x, items[i].y, 255, 0, 222);
			delete items[i];
			player.screenhold = 10;
			player.apples--;
			sfx.apple.play();
			flashbackground("#42ff00");
			player.hunger = 10;
			break;
		}
	}
}

function collides(type, a, b) {
	return window[type][a].x < b.x + b.w &&
		window[type][a].x + gridsize > b.x &&
		window[type][a].y < b.y + b.h &&
		window[type][a].y + gridsize > b.y;
}

//Help window
function openhelp(toggle) {
	var help = document.getElementById('help');
	if (toggle) {
		help.style.display = 'none';
	} else {
		help.style.display = 'block';
	}
}

window.onload = function() {
	gameport = document.getElementById('gameport').getContext('2d');
	gameport.canvas.width = canvassize.w;
	gameport.canvas.height = canvassize.h;

	background = document.getElementById('background').getContext('2d');
	background.canvas.width = canvassize.w;
	background.canvas.height = canvassize.h;

	ui = document.getElementById('ui').getContext('2d');
	ui.canvas.width = canvassize.w;
	ui.canvas.height = canvassize.h;

	//Help
	var help = document.getElementById('help');
	help.style.display = 'none';
	var helpbutton = document.getElementById('helpbutton');
	var closebutton = document.getElementById('closebutton');
	helpbutton.addEventListener("click", function(e) {
		openhelp();
	}, false);
	closebutton.addEventListener("click", function(e) {
		openhelp(true);
	}, false);

	//Nereast Neighbour - IE doesn't listen to this
	gameport.mozImageSmoothingEnabled = false;
	gameport.webkitImageSmoothingEnabled = false;
	gameport.imageSmoothingEnabled = false;
	gameport.imageSmoothingEnabled = false;
	background.mozImageSmoothingEnabled = false;
	background.webkitImageSmoothingEnabled = false;
	background.imageSmoothingEnabled = false;
	ui.mozImageSmoothingEnabled = false;
	ui.webkitImageSmoothingEnabled = false;
	ui.imageSmoothingEnabled = false;

	gameport.canvas.oncontextmenu = function() {
		return false;
	};

	document.addEventListener('keyup', function(e) {
		keystate[e.keyCode] = 0;
	}, false);
	document.addEventListener('keydown', function(e) {
		keystate[e.keyCode] = 1;
	}, false);

	ui.drawImage(logo, 0, 0, 41, 20, 236, 220, 328, 160);
	ui.drawImage(sprites, 32, 8, 40, 8, 236, 448, 328, 64);

	requestAnimFrame(rendering);
};